<?php
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// Copyright Author Dany De Bontridder danydb@aevalys.eu 21/11/24
/*! 
 * \file
 * \brief add selected operations to the list of payement
 * \code
 *
// received parameter example

 array (size=13)
  'act' => string 'payment_add_list' (length=16)
  'ac' => string 'COMPTA/MENUACH/XMLPY' (length=20)
  'gDossier' => string '74' (length=2)
  'plugin_code' => string 'XMLPY' (length=5)
  'first' => string '2' (length=1)
  'list_id' => string '5' (length=1)
  'mtable_id' => string 'tbl673f2a887e19d' (length=16)
  'upd_rec2' => string 'Ajouter la sélection  aux paiements' (length=36)
  'ledger_type' => string 'ALL' (length=3)
  'target' => string 'search_multi_operation_dg' (length=25)
  'jr_concerned7820' => string 'CEBO' (length=4)
  'jr_concerned7821' => string 'DIV' (length=3)
  'jr_concerned7819' => string 'ELECT' (length=5)
 * \endcode
 */
/**
 * @var $http HttpInput inherited from ajax.php
 * @var $cn Database inherited from ajax.php
 */
if (!defined('ALLOWED')) {
    die ("not allowed");
}

try {
    $list_id = $http->post('list_id', "number");
    $mtable_id = $http->post('mtable_id', "string");
} catch (\Exception $e) {
    echo $e->getMessage();
    return;
}

// retrieve jr_id from  jr_concernedXXX
$aKey = array_keys($_POST);

$operation_mtable = \Noalyss\XMLPayment\Operation_MTable::build(0, $list_id);
$operation_mtable->set_object_name($mtable_id);

try {
    $cn->start();
    foreach ($aKey as $key) {
        if (strpos("x" . $key, 'jr_concerned') == 1) {
            $jr_id = substr($key,12);
            if (isNumber($jr_id) == 1) {
                // retrieve info from Operation::make_info and insert it the list IF the operation is not
                // already in that list
                $info = \Noalyss\XMLPayment\Operation::make_info($jr_id);
                if ($info['status'] == 'OK') {
                    if ($cn->get_value('select count(*) from xmlpayment.payment_detail 
where d_jr_id=$1 and payment_id=$2', [$jr_id, $list_id]) == 0) {
                        $payment_detail = new \Noalyss\XMLPayment\Payment_Detail_SQL($cn);
                        $payment_detail->setp('d_amount', $info['amount']);
                        $payment_detail->setp('d_bank_account', $info['bank']);
                        $payment_detail->setp('fiche_id', $info['card_id']);
                        $payment_detail->setp('d_message', $info['msg']);
                        $payment_detail->setp('payment_id', $list_id);
                        $payment_detail->setp('d_jr_id', $jr_id);
                        $payment_detail->save();
                    }
                }
            }
        }
    }
    $cn->commit();

} catch (\Exception $e) {
    $cn->rollback();

}

$ret=$operation_mtable->execute_query(' where payment_id=$1',[$list_id] );
$nb=\Database::num_row($ret);

printf('<table class="result" id="tb%s">', $mtable_id);
$operation_mtable->display_table_header();
echo '<tbody>';
for ($i=0; $i<$nb; $i++)
{
    $row=Database::fetch_array($ret, $i);
    $operation_mtable->display_row($row);
}
echo '</tbody>';
echo "</table>";