<?php

namespace Noalyss\XMLPayment;

/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// Copyright Author Dany De Bontridder danydb@aevalys.eu 21/11/24
/*! 
 * \file
 * \brief Manage Accountany Operation and payment_detail
 */

class Operation
{
    /**
     * @brief operation , create a array to fill in the information in PAYMENT_DETAIL
     * @param $jrn_id
     * @return array key = status,bank, amount, qcode, msg , card_id
     * @throws \Exception
     */
    public static function make_info($jrn_id)
    {
        // connect to current folder
        $cn=\Dossier::connect();
        // retrieve bank message from JRN_INFO
        $bank_msg = $cn->get_value("select ji_value from jrn_info where jr_id=$1 and id_type='OTHER'", [$jrn_id]);

        // depending of the ledger, retrieve qcode CARD and also amount to pay
        $type = $cn->get_value("
    select jrn_def.jrn_def_type 
    from 
        jrn 
        join jrn_def on (jrn_def_id=jr_def_id) 
    where
        jr_id=$1", array($jrn_id));

        // qcode of the tiers
        $qcode = "";

        // amount
        $amount = 0;

        // fiche.f_id of the tiers
        $card_id = 0;

        // default bank account of the tiers
        $bank_account = "";

        switch ($type) {
            case 'ACH':
                $card_id = $cn->get_value('select distinct qp_supplier 
                from quant_purchase qp 
                    join jrn on (qp_internal=jr_internal)  
                where jr_id=$1', [$jrn_id]);

                $amount = $cn->get_value('
        select sum(qp_price+qp_vat+qp_nd_tva_recup+qp_nd_tva-qp.qp_vat_sided)
        from quant_purchase qp 
            join jrn on (qp_internal=jr_internal)  
        where jr_id=$1
        ', [$jrn_id]);

                break;
            case 'VEN':
                $card_id = $cn->get_value('select distinct qs_client 
            from quant_sold 
                join jrn on (qs_internal=jr_internal)  
            where jr_id=$1', [$jrn_id]);
                $amount = 0;
        }
// if card found then retrieve qcode and bank_account
        if ($card_id != 0) {
            $qcode = $cn->get_value("select ad_value from fiche_detail where ad_id=$1 and f_id=$2", [ATTR_DEF_QUICKCODE, $card_id]);
            $bank_account = $cn->get_value("select ad_value from fiche_detail where ad_id=$1 and f_id=$2", [ATTR_DEF_BQ_NO, $card_id]);
        }
        $answer = [];
        // if $type is empty , then the operation doesn't exist
        $answer['status'] = ($type != "") ? 'OK':'NOK';
        $answer['amount'] = $amount;
        $answer['qcode'] = $qcode;
        $answer['msg'] = $bank_msg;
        $answer['bank'] = $bank_account;
        $answer['card_id'] = $card_id;
        return $answer;
    }
}
