<?php
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// Copyright Author Dany De Bontridder danydb@aevalys.eu 13/09/24
/*! 
 * \file
 * \brief manage the operations for a list
 */
namespace Noalyss\XMLPayment;

class Operation_MTable extends \Manage_Table_SQL
{
    private $payment_id;

    public function __construct(Payment_Detail_SQL $p_table,$payment_id)
    {
        parent::__construct($p_table);
        $this->payment_id=$payment_id;

    }

    static function build($payemnt_detail_id,$payment_id):Operation_MTable
    {
        global $g_config_xmlpay;
        $payment_detail_sql=new Payment_Detail_SQL($g_config_xmlpay['cn'],$payemnt_detail_id);
        $payment_detail_sql->setp('payment_id',$payment_id);
        $operation_mtable=new Operation_MTable($payment_detail_sql, $payment_id);
        $operation_mtable->add_json_param("act", "operation_mtable");
        $operation_mtable->add_json_param("payment_id", $payment_id);
        $operation_mtable->add_json_param("plugin_code", $g_config_xmlpay['plugin_code']);
        $operation_mtable->add_json_param("ac", $g_config_xmlpay['ac']);
        $operation_mtable->add_json_param("dossier_id", $g_config_xmlpay['dossier_id']);

        $operation_mtable->set_order(['fiche_id','d_bank_account','d_amount','d_message','d_jr_id','d_error','tech_user','tech_date']);
        $operation_mtable->set_col_label('fiche_id', 'Fiche');
        $operation_mtable->set_col_label('d_bank_account', 'N°compte en banque');
        $operation_mtable->set_col_label('d_message', 'Communication');
        $operation_mtable->set_col_label('tech_user', 'Utilisateur');
        $operation_mtable->set_col_label('tech_date', 'Date saisie');
        $operation_mtable->set_col_label('d_jr_id', 'Opération');
        $operation_mtable->set_col_label('d_amount', 'Montant');
        $operation_mtable->set_col_label('d_error', 'Valide');

        $operation_mtable->set_col_type('d_amount', 'numeric');
        $operation_mtable->set_col_type("fiche_id", "custom");
        $operation_mtable->set_col_type("d_jr_id", "custom");
        $operation_mtable->set_col_type("d_error", "custom");
        $operation_mtable->set_col_type("d_bank_account", "text");
        $operation_mtable->set_icon_mod("left");
        $operation_mtable->set_property_updatable("tech_user", false);
        $operation_mtable->set_property_updatable("tech_date", false);
        return $operation_mtable;

    }
    function display_row_custom($p_key, $p_value, $p_id = 0)
    {
        $cn=$this->table->cn;
        if ( $p_key == 'fiche_id') {
            $qcode=$cn->get_value("select ad_value from fiche_detail where ad_id=$1 and f_id=$2"
                ,[ATTR_DEF_QUICKCODE,$p_value]);
            echo '<td>';

            echo \HtmlInput::card_detail($qcode);
            echo '</td>';
            return;
        }
        if ( $p_key == 'd_jr_id') {
            $p_mesg=$cn->get_value('select coalesce(jr_pj_number,jr_internal) from jrn where jr_id=$1',[$p_value]);

            echo '<td>';
            echo \HtmlInput::detail_op($p_value, $p_mesg);
            echo '</td>';
            return;
        }
        if ( $p_key == 'd_bank_account') {

            echo '<td>';
            echo h($p_value);
            echo '</td>';
            return;
        }
        if ($p_key == "d_error") {
            if ($p_value==0) {
                echo td("correct",'style="background-color:darkgreen;color:lightgreen;"');
                return;
            }
            //  CSS style for error
            $style='style="background-color:red;color:pink"';

            // CSS Style for warning
            $style_warning='style="background-color:orange;color:red"';

            // if $p_value == 90 then it is a warning , the column d_error_detail contains a message
            if ($p_value == 90) {

                $json=json_decode(
                    $cn->get_value("select d_error_detail from xmlpayment.payment_detail where d_id=$1", [$p_id])
                );
                if ( $json != NULL) {
                    echo td (sprintf ('virement similaire dans liste %s',$json->list),$style_warning);
                } else
                echo td($p_id,$style_warning);

                return;
            }

            echo match ($p_value ) {
                "1"=> td('Montant invalide',$style),
                "2"=> td('Fiche invalide',$style),
                "3"=> td('IBAN invalide',$style)

            };
        }
        parent::display_row_custom($p_key, $p_value,$p_id);

    }
    function input()
    {
        if ( $this->table->get("d_amount")=="") {
            $this->table->set("d_amount",0);
        }
        parent::input();
    }
    function input_custom($p_key, $p_value)
    {
        $cn=$this->table->cn;

        // for fiche_id, search for a Card , any card is usable
        if ( $p_key == 'fiche_id') {
            // Quick Code
            $quick_code=new \ICard('fiche_id');
            $quick_code->set_dblclick("fill_ipopcard(this);");


            // name of the field to update with the name of the card
            $quick_code->set_attribute('label', "label_fiche");

            // name of the field to update with the name of the card
            $quick_code->set_attribute('typecard', 'all');

            // xmlPayment created when the page is loaded , created in xmlpay_javascript.js
            $quick_code->set_function(sprintf('xmlPayment.fillData(\'%s\')',$quick_code->id));
            $quick_code->javascript=sprintf(' onchange="xmlPayment.fillData(\'%s\');" ',
                $quick_code->id);

            $quick_code->value=$p_value;

            $label='';
            if ($quick_code->value!='')
            {
                $Fiche=new \Fiche($cn,$p_value);
                $quick_code->value=$Fiche->get_quick_code();
                $label=$Fiche->strAttribut(ATTR_DEF_NAME);
            }

            echo $quick_code->input();
            echo \Icon_Action::button_magnifier(uniqid()   ,"xmlPayment.search_card_box()");
            echo span($label,' id="label_fiche"');


            return;
        }

        if ($p_key == 'd_jr_id') {
            // display a button to search an operation
            $input=new IConcerned('d_jr_id',value: $p_value);

            echo $input->input();
            $id=$this->table->getp('d_id');
            echo \HtmlInput::anchor_action(_("Complèter depuis opération"), "xmlPayment.copy_info_payment('{$id}')",p_class:"line");
            return;
        }

    }

    /**
     * @brief get value from FORM
     * @return void
     */
    function from_request()
    {
        parent::from_request(); // TODO: Change the autogenerated stub
        $http=new \HttpInput();
        $this->table->payment_id=$http->request('payment_id','number');
        if ( ! empty( $this->table->fiche_id) ) {
            $card=new \Fiche($this->table->cn);
            if ( $card->get_by_qcode(\noalyss_trim($this->table->fiche_id)) == 0 ) {
                // card found and replace by fiche.f_id
                $this->table->fiche_id=$card->id;
            } else {
                // card not found
                $this->table->fiche_id = null;
            }
        }

    }

    /**
     * @brief check the input
     * @return void
     */
    function check()
    {
        // amount > 0
        $payment_sql=$this->get_table();
        if ( $payment_sql->get('d_amount')<=0) {
            $this->set_error("d_amount", _('Montant positif'));
        }

        if ( \noalyss_trim($payment_sql->get('d_bank_account')) == "") {
            $this->set_error("d_bank_account", _('Aucun compte en banque donné'));
        }
        // a valid card if mandatory
        if ( $payment_sql->get('fiche_id') == "" )
        {
            $this->set_error("fiche_id", _('Aucune fiche donnée'));
        }
        // check that IBAN is valid
        $iban=$payment_sql->get('d_bank_account');
        if ( Tools::check_iban($iban )!= true) {

            $this->set_error("d_bank_account", _('Banque incorrecte'));
        }
        $payment_sql->set('d_bank_account',$iban);
        
        if ($this->count_error()==0) return true;
        return false;
    }

    /**
     * @brief set a warning if this bank transfer has a similar one in another list
     * @exception  if SQL fails
     * @return void
     */
    function update_warning() {
        global $g_config_xmlpay;
        
        /**
         * @var $list_id int  XMLPAYMENT.PAYMENT.P_ID
         */
        $list_id = $this->table->get('payment_id');
        /**
         * @var  $conx \Database connexion
         */
        $conx=$g_config_xmlpay['cn'];

        try {
        $conx->start();

        // disable trigger
        $conx->exec_sql('ALTER TABLE  xmlpayment.payment_detail DISABLE TRIGGER trg_set_tech_user');

        // clean all warnings
        $conx->exec_sql("update xmlpayment.payment_detail set d_error=0, d_error_detail=null    where payment_id=$1", [$list_id]);

        // look for duplicate with same jr_id

        $conx->exec_sql("
              with m as (
                        select d1.d_id current_id
                            ,d1.payment_id current_list
                            ,d2.d_id duplicate_list_detail 
                            ,d2.payment_id duplicate_list 
                        from 
                            xmlpayment.payment_detail d1
                        join xmlpayment.payment_detail d2
                        on (d1.d_jr_id=d2.d_jr_id)
                       where
                           d1.payment_id=$1
                           and d1.payment_id<>d2.payment_id)
                        update xmlpayment.payment_detail set d_error=90 ,
                                d_error_detail=format('{\"type\":\"DUP\",\"payment_id\":%s,\"list\":%s}',m.duplicate_list_detail,m.duplicate_list)
                                from m
                                where 
                                d_id=m.current_id
                        ", [$list_id]);

        // look for duplicate with same message and amount and tiers
        $conx->get_array("
              with m as (
                        select d1.d_id current_id
                            ,d1.payment_id current_list
                            ,d2.d_id duplicate_list_detail 
                            ,d2.payment_id duplicate_list 
                        from 
                            xmlpayment.payment_detail d1
                        join xmlpayment.payment_detail d2
                        on (d1.fiche_id=d2.fiche_id and d1.d_amount=d2.d_amount and d1.d_message=d2.d_message)
                       where
                           d1.payment_id=$1
                           and d1.payment_id<>d2.payment_id
                           and d1.d_jr_id is null
                           and d2.d_jr_id is null
                           )
                        update xmlpayment.payment_detail set d_error=90 ,
                                d_error_detail=format('{\"type\":\"DUP\",\"payment_id\":%s,\"list\":%s}',m.duplicate_list_detail,m.duplicate_list)
                                from m
                                where 
                                d_id=m.current_id
                        ", [$list_id]);
            // disable trigger
            $conx->exec_sql('ALTER TABLE  xmlpayment.payment_detail ENABLE TRIGGER trg_set_tech_user');
            $conx->commit();
        } catch (\Exception $e) {
            $conx->rollback();
            throw $e;
        }

    }

    function display_table($p_order = "", $p_array = NULL)
    {
        // check for duplicate , set a warning : code d_error = 90
        $this->update_warning();

        parent::display_table($p_order, $p_array); // TODO: Change the autogenerated stub
    }
}