<?php

namespace Noalyss\XMLPayment;

/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// Copyright Author Dany De Bontridder danydb@aevalys.eu 4/10/24
/*! 
 * \file
 * \brief compute the XML Sepa file to be given to the bank for payment
 */
const ERROR_SEPA=[0=>'OK',1=>'Montant invalide',2=>'Fiche invalide',3=>'IBAN incorrect',99=>'Autre error'];

class Sepa
{
    protected $payment_id; //!< int pk of xmlpayment.payment
    protected $domdoc; //!< DomDocument 1.0
    protected $dbcnx; //!< \Database database connx


    function __construct($payment_id)
    {
        $this->payment_id = $payment_id;
        $this->domdoc = null;
        $this->dbcnx=\Dossier::connect();

    }

    /**
     * @brief make XML For initiatingParty (InitgPty)
     * @return void
     */
    protected function build_initiatingParty()
    {
        global $g_parameter;
        $initgPty = $this->domdoc->createElement('InitgPty');
        $initgPty->appendChild($this->domdoc->createElement('Nm', $g_parameter->MY_NAME));
        return $initgPty;

    }

    /**
     * @brief build GroupHeader <GrpHdr)
     * @return void
     */
    private function append_GroupHeader($xml_document)
    {
        $group_header = $this->domdoc->createElement('GrpHdr');
        // Add MsgId
        $group_header->appendChild($this->domdoc->createElement('MsgId', 'noalyss'
            .sprintf("/%s/%s/",$this->payment_id,\Dossier::id())
            . date('ymdHi')));
        // Creation Date
        $group_header->appendChild($this->domdoc->createElement('CreDtTm', date('c')));

        // NB Transaction
        $nb_transaction = $this->dbcnx->get_value('select count(*) from xmlpayment.payment_detail where payment_id=$1'
            , [$this->payment_id]);
        $group_header->appendChild($this->domdoc->createElement('NbOfTxs', $nb_transaction));

        // Sum Amount
        $sum_amount = $this->dbcnx->get_value('select sum(d_amount) from xmlpayment.payment_detail where payment_id=$1'
            , [$this->payment_id]);

        $group_header->appendChild($this->domdoc->createElement('CtrlSum', sprintf('%.2f', $sum_amount)));

        $initiatingParty = $this->build_initiatingParty();
        $group_header->appendChild($initiatingParty);
        $xml_document->appendChild($group_header);

    }

    /**
     * @brief build PaymentInformation (PmtInf)
     * @return DOMElement
     */
    private function build_PaymentInformation():\DOMElement
    {
        global $g_parameter;
        $xml_document = $this->domdoc->createElement('PmtInf');
        $xml_document->appendChild($this->domdoc->createElement('PmtInfId', uniqid('noalyss/')));
        $xml_document->appendChild($this->domdoc->createElement('PmtMtd', 'TRF'));
        $xml_document->appendChild($this->domdoc->createElement('BtchBookg', 'false'));




        //$xml_document->appendChidl()

        //-------------------------------------------------------------------------------------------
        //PaymentTypeInformation <PmtTpInf>
        //-------------------------------------------------------------------------------------------
        $tmpTpInf = $this->domdoc->createElement('PmtTpInf');

        $InstrPrty = $this->domdoc->createElement('InstrPrty', 'NORM');

        $SvcLvl = $this->domdoc->createElement('SvcLvl');
        $Cd = $this->domdoc->createElement('Cd', 'SEPA');
        $SvcLvl->appendChild($Cd);

        $tmpTpInf->appendChild($InstrPrty);
        $tmpTpInf->appendChild($SvcLvl);
        $xml_document->appendChild($tmpTpInf);
        //-------------------------------------------------------------------------------------------
        //Execution date : today
        //-------------------------------------------------------------------------------------------
        // exec_date == value from XMLPAYMENT.PAYMENT.P_EXEC_DATE
        $exec_date=$this->dbcnx->get_value("select to_char(p_exec_date,'YYYY-MM-DD')  
        from xmlpayment.payment where p_id=$1",
            [$this->payment_id]);
        // if exec_date == null then exec_date == today
        if ( $exec_date == "" ) {
            $exec_date=date('Y-m-d');
        } else{
            // if exec_date < today then exec_date == today
            $d=\DateTime::createFromFormat("Y-m-d", $exec_date);
            if ($d ==false || $d->format('Ymd')< date('Ymd')) {
                $exec_date=date('Y-m-d');
            }
        }

        $xml_document->appendChild($this->domdoc->createElement('ReqdExctnDt', $exec_date));
        //-------------------------------------------------------------------------------------------
        // Add debitor
        //-------------------------------------------------------------------------------------------
        $debitor = $this->domdoc->createElement('Dbtr');
        $debitor->appendChild($this->domdoc->createElement('Nm', $g_parameter->MY_NAME));
        $xml_document->appendChild($debitor);

        //-------------------------------------------------------------------------------------------
        // add Debitor Account
        //-------------------------------------------------------------------------------------------
        $account = $this->dbcnx->get_row('
            select b_id
                 ,b_bank_number
                 ,b_bank_id 
            from 
                xmlpayment.bank_account 
                    join xmlpayment.payment on (bank_account.b_id = payment.bank_account_id)
            where 
                p_id=$1', [$this->payment_id]);
        if (count($account) == 0) throw new \Exception(__FUNCTION__ . "not bank account given");

        $debitor_account = $this->domdoc->createElement('DbtrAcct');
        $id= $this->domdoc->createElement('Id');
        $id->appendChild($this->domdoc->createElement('IBAN',$account['b_bank_number']));
        $debitor_account->appendChild($id);
        $debitor_account->appendChild($this->domdoc->createElement('Ccy','EUR'));
        $xml_document->appendChild($debitor_account);

        //-------------------------------------------------------------------------------------------
        // <DbtrAgt>
        //-------------------------------------------------------------------------------------------
        $debitor_agent=$this->domdoc->createElement('DbtrAgt');
        $financial=$this->domdoc->createElement('FinInstnId');
        $financial->appendChild($this->domdoc->createElement('BIC',$account['b_bank_id']));
        $debitor_agent->appendChild($financial);
        $xml_document->appendChild($debitor_agent);

        return $xml_document;

    }

    /**
     * @brief add the list of payment to PmtInf payment_information
     * @param $payment_information DOMElement
     * @return void
     * @throws \Exception if list empty
     */
    private function append_CreditTransferTransactionInformation($payment_information)
    {

        $a_payment=$this->dbcnx->get_array('
                select d_id
                     ,d_amount
                     ,d_bank_account
                     ,d_message
                     , fiche_id
                     ,d_jr_id 
                from 
                    xmlpayment.payment_detail 
                where
                    payment_id=$1 ',[$this->payment_id]);
        $nb_payement=count($a_payment);
        if ($nb_payement==0) {
            throw new \Exception(__FUNCTION__ . 'Liste paiement vide');
        }
        for ($i=0;$i<$nb_payement;$i++)
        {
            $xml_document=$this->domdoc->createElement('CdtTrfTxInf');
            $id=$this->domdoc->createElement('PmtId');

            // Identification operation  pattern id:(xmlpayment.d_id)/jr:(jrn.jr_id)
            $id->appendChild($this->domdoc->createElement('EndToEndId'
                ,sprintf("id:%s/jr:%s",$a_payment[$i]['d_id'],$a_payment[$i]['d_jr_id'])));
            $xml_document->appendChild($id);
            $amt=$this->domdoc->createElement('Amt');
            $instdAmt=$this->domdoc->createElement('InstdAmt',sprintf("%.2f",$a_payment[$i]['d_amount']));
            $instdAmt->setAttribute('Ccy','EUR');
            $amt->appendChild($instdAmt);
            $xml_document->appendChild($amt);
            // add name
            $card=new \Fiche($this->dbcnx,$a_payment[$i]['fiche_id']);
            $cdtr=$this->domdoc->createElement('Cdtr');
            $cdtr->appendChild($this->domdoc->createElement('Nm',$card->strAttribut(ATTR_DEF_NAME)));
            $xml_document->appendChild($cdtr);

            // add bank account
            $CdtrAcct=$this->domdoc->createElement('CdtrAcct');
            $id=$this->domdoc->createElement('Id');
            $id->appendChild($this->domdoc->createElement('IBAN',$a_payment[$i]['d_bank_account']));
            $CdtrAcct->appendChild($id);
            $xml_document->appendChild($CdtrAcct);

            // add a message is not empty
            if ( $a_payment[$i]['d_message'] != "")
            {
                    $RmtInf=$this->domdoc->createElement('RmtInf');
                    $RmtInf->appendChild($this->domdoc->createElement('Ustrd',$a_payment[$i]['d_message']??''));
                    $xml_document->appendChild($RmtInf);
            }

            $payment_information->appendChild($xml_document);


        }
    }

    /**
     * @brief returns the XML SEPA file
     * @return \DOMDocument
     * @throws \DOMException
     */
    function build_xml():\DOMDocument
    {
        // create root <Document>
        $this->domdoc = new \DOMDocument('1.0', 'UTF-8');
        $this->domdoc->formatOutput = true;
        $document = $this->domdoc->createElementNS('urn:iso:std:iso:20022:tech:xsd:pain.001.001.03', 'Document');
        $document->setAttribute('xmlns:xsi', 'http://www.w3.org/2001/XMLSchema-instance');
        // add  <CstmrCdtTrfInitn>
        $CstmrCdtTrfInitn = $this->domdoc->createElement('CstmrCdtTrfInitn');

        // add  <GrpHdr>
        $this->append_GroupHeader($CstmrCdtTrfInitn);


        // add  <PmtInf> :

        $payment_information = $this->build_PaymentInformation();
        
        // add <CdtTrfTxInf> list of payment
        $this->append_CreditTransferTransactionInformation($payment_information);
        $CstmrCdtTrfInitn->appendChild($payment_information);

        $document->appendChild($CstmrCdtTrfInitn);


        // add document to the root
        $this->domdoc->appendChild($document);


        return $this->domdoc;
    }

    /**
     * @brief check that all data are correct
     * it returns an  array
     *
     * keys:
     *    - amount (nb of invalid amount),
     *    - card (nb of invalid cards),
     *    - iban (nb of invalid iban),
     *    - record (nb of valid records),
     * @return array
     */
    function verify()
    {
        /// reset error
        $this->dbcnx->exec_sql('update xmlpayment.payment_detail 
           set d_error=0
        where payment_id=$1',[$this->payment_id]);


        /// Amount > 0
        $this->dbcnx->exec_sql('update xmlpayment.payment_detail 
           set d_error=1
        where d_amount <= 0 and payment_id=$1',[$this->payment_id]);

       $nb_zero=$this->dbcnx->get_value('select count(*) from xmlpayment.payment_detail where payment_id=$1 and d_error=1',[$this->payment_id]);
        /// Card must exist
        $this->dbcnx->exec_sql('update xmlpayment.payment_detail 
           set d_error=2
        where fiche_id is null and payment_id=$1',[$this->payment_id]);

        $nb_card=$this->dbcnx->get_value('select count(*) from xmlpayment.payment_detail where payment_id=$1 and d_error=2',[$this->payment_id]);

        /// IBAN valid Creditor and Debitor
        // get all data from list payment
        $aPayment_detail = $this->dbcnx->get_array("select * from xmlpayment.payment_detail where d_error=0 and payment_id=$1",[$this->payment_id]);
        $nb_iban=0;
        if (count($aPayment_detail) >0 )
        {
            foreach ($aPayment_detail as $payment_detail ) {
                if ( Tools::check_iban($payment_detail['d_bank_account']) == false ) {
                    $this->dbcnx->exec_sql("update xmlpayment.payment_detail set d_error = 3 where d_id=$1",[$payment_detail['d_id']]);
                    $nb_iban++;
                }
            }
        }
        /// At least one record in payement
        $nb_record=$this->dbcnx->get_value('select count(*) from xmlpayment.payment_detail where payment_id=$1 and d_error=0',[$this->payment_id]);

        /// return array('amount'=>$nb_zero,'card'=>$nb_card,'iban'=>$nb_iban,'valid'=>$nb_record);
        return array('amount'=>$nb_zero,'card'=>$nb_card,'iban'=>$nb_iban,'valid'=>$nb_record);

    }
}