<?php

/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Revision$ */

// Copyright (c) 2024 Author Dany De Bontridder dany@alchimerys.be

/**
 * @file
 * @brief Utility for IBAN
 */
namespace Noalyss\XMLPayment;

class Tools
{
    /**
     * @brief
     * @return void
     */
    public static function compute_letter_value()
    {
        global $aLetter,$aLetterValue;

        static $make_string=null;
        if ( $make_string == null ) {
            for ($i=65;$i!=91;$i++) {
                $make_string[chr($i)]=$i-55;
            }
            $aLetter=array_keys($make_string);
            $aLetterValue=array_values($make_string);
        }
    }

    /**
     * @brief check that an IBAN is valid
     * @param $iban string
     * @return bool false the IBAN is invalid, true IBAN is VALID
     */
    public static function check_iban(&$iban):bool
    {
        global $aLetter,$aLetterValue;
        if (trim($iban??"") == "") return false;

        self::compute_letter_value();

        $iban=strtoupper($iban);
        str_replace([" ",",",".","-"],'', $iban);

        $first=substr($iban, 0,4);
        $chain=substr($iban,4).$first;

        $replaced=str_replace($aLetter,$aLetterValue,$chain);

        // computed by slice of 10: mod function is limited
        $start=0;
        $slice=10;
        $result="";
        $length=strlen($replaced);
        while ($start < $length)
        {
            $slice_string=$result.substr($replaced,$start,$slice);
            $result=$slice_string%97;
            $start+=$slice;
        }

        if ($result == 1) return true;

        return false;
    }
}