<?php
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/* $Revision$ */

// Copyright (c) 2023 Author Dany De Bontridder dany@alchimerys.be
/*
/*!\file
 * \brief main file for : change
 *   - skel with code of the plugin
 *   - Route n : with real name
 * Global variable are defined into constant.php , by default
 *   - $g_config_xmlpay array of info (dir : plugin's directory,cn : Database conx, ac , plugin_code,dossier_id)
 */
/*!
 * \mainpage XMPY
 *
 * \section Introduction
 *
Cette extension permet de composer des listes de virements , qui suivent le format SEPA. Cette norme européenne est acceptée par la plupart des banques.


Voir aussi documentation dans doc/xml_payment.adoc

 */
if (!defined('ALLOWED'))
    die('Appel direct ne sont pas permis');
require_once 'xmlpay_constant.php';
global $g_config_xmlpay;

$aExtension=Extension::read_definition($g_config_xmlpay['dir']."/plugin.xml");

\Extension::check_version($aExtension[0]->noalyss_version);

$http = new HttpInput();



ob_start();
require_once 'xmlpay_javascript.js';
$j = ob_get_contents();
ob_end_clean();
echo create_script($j);

$url = "?" . http_build_query(["gDossier" => $g_config_xmlpay['dossier_id'],
        "ac" => $g_config_xmlpay['ac'],
        "plugin_code" => $g_config_xmlpay['plugin_code']]);


$sa = $http->request("sa", "string", "none");

$cn = Dossier::connect();
if ($cn->exist_schema('xmlpayment') == false) {

    $iplugn = new Noalyss\XMLPayment\Install_Plugin($cn);
    $iplugn->install();
    $def = 2;
}
$version = $cn->get_value('select max(v_id) from xmlpayment.version');

if ( $version < XMLPAYMENT_VERSION) {

    $iplugn = new Noalyss\XMLPayment\Install_Plugin($cn);
    $iplugn->upgrade(XMLPAYMENT_VERSION);
}

$script=<<<EOF
var dossier_id='{$g_config_xmlpay['dossier_id']}';
var access_code='{$g_config_xmlpay['ac']}';
var plugin_code='{$g_config_xmlpay['plugin_code']}';
EOF;

echo create_script($script);

if ($cn->get_value('select max(v_id) from xmlpayment.version') < XMLPAYMENT_VERSION) {
    $iplugn = new Noalyss\XMLPayment\Install_Plugin($cn);
    $iplugn->upgrade(p_version: XMLPAYMENT_VERSION);
}
// show menu
echo '<div style="float:right"><a id="help"  href="http://wiki.noalyss.eu/doku.php?id=xmlpy" target="_blank">Aide</a>' .
    '<span style="font-size:0.8em;color:#9e5252;display:block">version:'.
    $aExtension['0']->version.
    '</span>' .
    '</div>';
$sa=$http->get("sa","string","list");

/**************************************************************************
 * List all SEPA
 *
 *************************************************************************/
echo '<div class="content" style="margin-top:5px;margin-left:5px">';
if ($sa == "list") {
    \Noalyss\XMLPayment\List_Operation::display_list();
}

/**************************************************************************
 * detail of a list or a new list
 *
 *************************************************************************/
if ($sa=="list_detail") {

    // if $sb == -1 create an empty list
    $p_id=$http->request("p_id","number");
    if ( $p_id == -1) {
        $payment=new \Noalyss\XMLPayment\Payment_SQL($g_config_xmlpay['cn'],-1);
        $payment->set("p_status",'N');

        $payment->save();
        $p_id=$payment->get("p_id");
    }
    echo h1("Liste n° $p_id",'class="h-section"');
    echo h2("Donneur d'ordre",'class="h-section"');
    //-------------------------------------------------------------------------------------------
    //
    // Enveloppe société compte donneur d'ordre
    //-------------------------------------------------------------------------------------------
    $list_operation=new \Noalyss\XMLPayment\List_Operation(new \Noalyss\XMLPayment\Payment_SQL($cn,$p_id));
    $list_operation->select();
    echo h2("Liste des paiements ",'class="h-section"');

    $operation_mtable=\Noalyss\XMLPayment\Operation_MTable::build(-1,$p_id);
    $operation_mtable->set_object_name("payment_detail_mtable");
    $operation_mtable->display_table(' where payment_id = $1',[$p_id]);
    $operation_mtable->create_js_script();

    // Button for adding several operation from accountancy
    $js=sprintf("onclick=\"xmlPayment.showSearch_multi_operation('%s','%s')\"",
        $p_id,
        $operation_mtable->get_object_name()
    );
    echo \HtmlInput::button(uniqid(), _("Recherche d'opération à ajouter"),$js);

}
echo '</div>';
html_page_stop();
